@@ -64,7 +64,7 @@
  *   which allows describing the key frames for objects.</para>
  *   <para>The "properties" property has the following syntax:</para>
  *   <informalexample>
- *     <programlisting>
+ *     <programlisting><![CDATA[
  *  {
  *    "properties" : [
  *      {
@@ -77,7 +77,7 @@
  *        ]
  *    ]
  *  }
- *     </programlisting>
+ *     ]]></programlisting>
  *   </informalexample>
  *   <example id="ClutterAnimator-script-example">
  *     <title>ClutterAnimator definition</title>
@@ -88,7 +88,7 @@
  *     from its current position to the 100, 100 position in 20 percent
  *     of the duration of the animation; the second will using a cubic
  *     easing to move the actor to the 200, 200 coordinates.</para>
- *     <programlisting>
+ *     <programlisting><![CDATA[
  *  {
  *    "type" : "ClutterAnimator",
  *    "duration" : 1000,
@@ -113,7 +113,7 @@
  *      }
  *    ]
  *  }
- *     </programlisting>
+ *     ]]></programlisting>
  *   </example>
  * </refsect2>
  *
@@ -185,9 +185,13 @@ enum
   PROP_0,
 
   PROP_DURATION,
-  PROP_TIMELINE
+  PROP_TIMELINE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 static void clutter_scriptable_init (ClutterScriptableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterAnimator,
@@ -440,17 +444,11 @@ clutter_animator_finalize (GObject *object)
   g_list_free (priv->score);
   priv->score = NULL;
 
-#if 0
-  for (; priv->score;
-       priv->score = g_list_remove (priv->score, priv->score->data))
-    {
-      clutter_animator_key_free (priv->score->data);
-    }
-#endif
-
   g_object_unref (priv->timeline);
   g_object_unref (priv->slave_timeline);
 
+  g_hash_table_destroy (priv->properties);
+
   G_OBJECT_CLASS (clutter_animator_parent_class)->finalize (object);
 }
 
@@ -1086,7 +1084,9 @@ clutter_animator_get_timeline (ClutterAnimator *animator)
  * Start the ClutterAnimator, this is a thin wrapper that rewinds
  * and starts the animators current timeline.
  *
- * Return value: the #ClutterTimeline that drives the animator.
+ * Return value: (transfer none): the #ClutterTimeline that drives
+ *   the animator. The returned timeline is owned by the #ClutterAnimator
+ *   and it should not be unreferenced
  *
  * Since: 1.2
  */
@@ -1354,7 +1354,7 @@ clutter_animator_set_key (ClutterAnimator *animator,
  * Returns a list of pointers to opaque structures with accessor functions
  * that describe the keys added to an animator.
  *
- * Return value: (transfer container) (element-type ClutterAnimatorKey): a
+ * Return value: (transfer container) (element-type Clutter.AnimatorKey): a
  *   list of #ClutterAnimatorKey<!-- -->s; the contents of the list are owned
  *   by the #ClutterAnimator, but you should free the returned list when done,
  *   using g_list_free()
@@ -1380,7 +1380,7 @@ clutter_animator_get_keys (ClutterAnimator *animator,
       ClutterAnimatorKey *key = k->data;
 
       if ((object == NULL || (object == key->object)) &&
-          (property_name == NULL || ((property_name == key->property_name))) &&
+          (property_name == NULL || (property_name == key->property_name)) &&
           (progress < 0  || fabs (progress - key->progress) < PROGRESS_EPSILON))
         {
           keys = g_list_prepend (keys, key);
@@ -1407,7 +1407,8 @@ clutter_animator_remove_key_internal (ClutterAnimator *animator,
 
   priv = animator->priv;
 
-  for (k = priv->score; k != NULL; k = k?k->next:NULL)
+again:
+  for (k = priv->score; k != NULL; k = k->next)
     {
       ClutterAnimatorKey *key = k->data;
 
@@ -1439,8 +1440,8 @@ clutter_animator_remove_key_internal (ClutterAnimator *animator,
                   next_key->ease_in = key->ease_in;
                 }
             }
-
-          k = priv->score = g_list_remove (priv->score, key);
+          priv->score = g_list_remove (priv->score, key);
+          goto again;
         }
     }
 
@@ -1449,7 +1450,7 @@ clutter_animator_remove_key_internal (ClutterAnimator *animator,
       GHashTableIter iter;
       gpointer key, value;
 
-again:
+again2:
       g_hash_table_iter_init (&iter, priv->properties);
       while (g_hash_table_iter_next (&iter, &key, &value))
         {
@@ -1457,7 +1458,7 @@ again:
           if (prop_actor_key->object == object)
             {
               g_hash_table_remove (priv->properties, key);
-              goto again;
+              goto again2;
             }
         }
     }
@@ -1538,7 +1539,7 @@ parse_animator_property (JsonArray *array,
   GObjectClass *klass;
   GParamSpec *pspec;
   GSList *valid_keys = NULL;
-  GList *k;
+  GList *array_keys, *k;
   ClutterInterpolation interpolation = CLUTTER_INTERPOLATION_LINEAR;
   gboolean ease_in = FALSE;
 
@@ -1610,9 +1611,8 @@ parse_animator_property (JsonArray *array,
   else
     g_value_init (clos->value, G_TYPE_POINTER);
 
-  for (k = json_array_get_elements (keys);
-       k != NULL;
-       k = k->next)
+  array_keys = json_array_get_elements (keys);
+  for (k = array_keys; k != NULL; k = k->next)
     {
       JsonNode *node = k->data;
       JsonArray *key = json_node_get_array (node);
@@ -1651,6 +1651,8 @@ parse_animator_property (JsonArray *array,
       valid_keys = g_slist_prepend (valid_keys, animator_key);
     }
 
+  g_list_free (array_keys);
+
   g_value_set_pointer (clos->value, g_slist_reverse (valid_keys));
 
   clos->result = TRUE;
@@ -1782,11 +1784,12 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_uint ("duration",
-                             "Duration",
-                             "The duration of the animation",
+                             P_("Duration"),
+                             P_("The duration of the animation"),
                              0, G_MAXUINT,
                              2000,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 
   /**
@@ -1798,10 +1801,11 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_object ("timeline",
-                               "Timeline",
-                               "The timeline of the animation",
+                               P_("Timeline"),
+                               P_("The timeline of the animation"),
                                CLUTTER_TYPE_TIMELINE,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_TIMELINE] = pspec;
   g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
 }
 
@@ -1820,7 +1824,6 @@ clutter_animator_init (ClutterAnimator *animator)
   clutter_animator_set_timeline (animator, clutter_timeline_new (2000));
 
   priv->slave_timeline = clutter_timeline_new (10000);
-  g_object_ref_sink (priv->slave_timeline);
 }
 
 
@@ -2114,23 +2117,38 @@ gboolean
 clutter_animator_key_get_value (const ClutterAnimatorKey *key,
                                 GValue                   *value)
 {
-  GType gtype;
-
   g_return_val_if_fail (key != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+  g_return_val_if_fail (G_VALUE_TYPE (value) != G_TYPE_INVALID, FALSE);
 
-  gtype = G_VALUE_TYPE (&key->value);
-
-  if (g_value_type_compatible (gtype, G_VALUE_TYPE (value)))
+  if (!g_type_is_a (G_VALUE_TYPE (&key->value), G_VALUE_TYPE (value)))
     {
-      g_value_copy (&key->value, value);
-      return TRUE;
-    }
+      if (g_value_type_compatible (G_VALUE_TYPE (&key->value),
+                                   G_VALUE_TYPE (value)))
+        {
+          g_value_copy (&key->value, value);
+          return TRUE;
+        }
 
-  if (g_value_type_transformable (gtype, G_VALUE_TYPE (value)))
-    {
-      if (g_value_transform (&key->value, value))
-        return TRUE;
-    }
+      if (g_value_type_transformable (G_VALUE_TYPE (&key->value),
+                                      G_VALUE_TYPE (value)))
+        {
+          if (g_value_transform (&key->value, value))
+            return TRUE;
+        }
 
-  return FALSE;
+      g_warning ("%s: Unable to convert from %s to %s for the "
+                 "property '%s' of object %s in the animator key",
+                 G_STRLOC,
+                 g_type_name (G_VALUE_TYPE (&key->value)),
+                 g_type_name (G_VALUE_TYPE (value)),
+                 key->property_name,
+                 G_OBJECT_TYPE_NAME (key->object));
+
+      return FALSE;
+    }
+  else
+    g_value_copy (&key->value, value);
+
+  return TRUE;
 }
