@@ -60,6 +60,7 @@
  * constraint = clutter_bind_constraint_new (rect[0], CLUTTER_BIND_X, 0.0);
  * clutter_actor_add_constraint_with_name (rect[2], "blue-x", constraint);
  * constraint = clutter_bind_constraint_new (rect[0], CLUTTER_BIND_Y, 0.0);
+ * clutter_actor_add_constraint_with_name (rect[2], "blue-y", constraint);
  * ]|
  *
  * The following code animates the second and third rectangles to "expand"
@@ -116,9 +117,13 @@ enum
 
   PROP_SOURCE,
   PROP_COORDINATE,
-  PROP_OFFSET
+  PROP_OFFSET,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterBindConstraint,
                clutter_bind_constraint,
                CLUTTER_TYPE_CONSTRAINT);
@@ -266,11 +271,12 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_object ("source",
-                               "Source",
-                               "The source of the binding",
+                               P_("Source"),
+                               P_("The source of the binding"),
                                CLUTTER_TYPE_ACTOR,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT);
+  obj_props[PROP_SOURCE] = pspec;
   g_object_class_install_property (gobject_class, PROP_SOURCE, pspec);
 
   /**
@@ -281,12 +287,13 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_enum ("coordinate",
-                             "Coordinate",
-                             "The coordinate to bind",
+                             P_("Coordinate"),
+                             P_("The coordinate to bind"),
                              CLUTTER_TYPE_BIND_COORDINATE,
                              CLUTTER_BIND_X,
                              CLUTTER_PARAM_READWRITE |
                              G_PARAM_CONSTRUCT);
+  obj_props[PROP_COORDINATE] = pspec;
   g_object_class_install_property (gobject_class, PROP_COORDINATE, pspec);
 
   /**
@@ -297,12 +304,13 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_float ("offset",
-                              "Offset",
-                              "The offset in pixels to apply to the binding",
+                              P_("Offset"),
+                              P_("The offset in pixels to apply to the binding"),
                               -G_MAXFLOAT, G_MAXFLOAT,
                               0.0f,
                               CLUTTER_PARAM_READWRITE |
                               G_PARAM_CONSTRUCT);
+  obj_props[PROP_OFFSET] = pspec;
   g_object_class_install_property (gobject_class, PROP_OFFSET, pspec);
 }
 
@@ -390,7 +398,7 @@ clutter_bind_constraint_set_source (ClutterBindConstraint *constraint,
       update_actor_coords (constraint);
     }
 
-  g_object_notify (G_OBJECT (constraint), "source");
+  _clutter_notify_by_pspec (G_OBJECT (constraint), obj_props[PROP_SOURCE]);
 }
 
 /**
@@ -433,7 +441,7 @@ clutter_bind_constraint_set_coordinate (ClutterBindConstraint *constraint,
 
   update_actor_coords (constraint);
 
-  g_object_notify (G_OBJECT (constraint), "coordinate");
+  _clutter_notify_by_pspec (G_OBJECT (constraint), obj_props[PROP_COORDINATE]);
 }
 
 /**
@@ -477,7 +485,7 @@ clutter_bind_constraint_set_offset (ClutterBindConstraint *constraint,
 
   update_actor_coords (constraint);
 
-  g_object_notify (G_OBJECT (constraint), "offset");
+  _clutter_notify_by_pspec (G_OBJECT (constraint), obj_props[PROP_OFFSET]);
 }
 
 /**
