@@ -156,6 +156,7 @@ static const GDebugKey clutter_debug_keys[] = {
   { "texture", CLUTTER_DEBUG_TEXTURE },
   { "event", CLUTTER_DEBUG_EVENT },
   { "paint", CLUTTER_DEBUG_PAINT },
+  { "pick", CLUTTER_DEBUG_PICK },
   { "gl", CLUTTER_DEBUG_GL },
   { "alpha", CLUTTER_DEBUG_ALPHA },
   { "behaviour", CLUTTER_DEBUG_BEHAVIOUR },
@@ -264,34 +265,6 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
     }
 }
 
-void
-_clutter_stage_maybe_setup_viewport (ClutterStage *stage)
-{
-  if ((CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_SYNC_MATRICES) &&
-      !CLUTTER_STAGE_IN_RESIZE (stage))
-    {
-      ClutterPerspective perspective;
-      gfloat width, height;
-
-      clutter_actor_get_preferred_size (CLUTTER_ACTOR (stage),
-                                        NULL, NULL,
-                                        &width, &height);
-      clutter_stage_get_perspective (stage, &perspective);
-
-      CLUTTER_NOTE (PAINT,
-                    "Setting up the viewport { w:%.2f, h:%.2f }",
-                    width, height);
-
-      _cogl_setup_viewport (width, height,
-                            perspective.fovy,
-                            perspective.aspect,
-                            perspective.z_near,
-                            perspective.z_far);
-
-      CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_SYNC_MATRICES);
-    }
-}
-
 void
 _clutter_do_redraw (ClutterStage *stage)
 {
@@ -301,6 +274,13 @@ _clutter_do_redraw (ClutterStage *stage)
 
   ctx = _clutter_context_get_default ();
 
+  /* We have an optimization to avoid pick renders while scene is static.
+   * This is implied by seeing multiple picks per frame so we have to
+   * reset a pick counter each frame and also invalidate any current pick
+   * buffer. */
+  ctx->picks_per_frame = 0;
+  ctx->have_complete_pick_buffer = FALSE;
+
   /* Before we can paint, we have to be sure we have the latest layout */
   _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
 
@@ -577,6 +557,7 @@ _clutter_do_pick (ClutterStage   *stage,
   guint32             id;
   GLboolean           dither_was_on;
   ClutterActor       *actor;
+  gboolean            is_clipped;
   CLUTTER_STATIC_COUNTER (do_pick_counter,
                           "_clutter_do_pick counter",
                           "Increments for each full pick run",
@@ -604,7 +585,7 @@ _clutter_do_pick (ClutterStage   *stage,
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
 
-  if (clutter_debug_flags & CLUTTER_DEBUG_NOP_PICKING)
+  if (G_UNLIKELY (clutter_pick_debug_flags & CLUTTER_DEBUG_NOP_PICKING))
     return CLUTTER_ACTOR (stage);
 
 #ifdef CLUTTER_ENABLE_PROFILE
@@ -617,16 +598,61 @@ _clutter_do_pick (ClutterStage   *stage,
 
   context = _clutter_context_get_default ();
 
+  /* It's possible that we currently have a static scene and have renderered a
+   * full, unclipped pick buffer. If so we can simply continue to read from
+   * this cached buffer until the scene next changes. */
+  if (context->have_complete_pick_buffer)
+    {
+      CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
+      cogl_read_pixels (x, y, 1, 1,
+                        COGL_READ_PIXELS_COLOR_BUFFER,
+                        COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                        pixel);
+      CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_read);
+
+      CLUTTER_NOTE (PICK, "Reusing pick buffer from previous render to fetch "
+                    "actor at %i,%i", x, y);
+
+      /* FIXME: This is a lazy copy and paste of the logic at the end of this
+       * function used when we actually do a pick render. It should be
+       * consolidated somehow.
+       */
+      if (pixel[0] == 0xff && pixel[1] == 0xff && pixel[2] == 0xff)
+        {
+          actor = CLUTTER_ACTOR (stage);
+          goto result;
+        }
+
+      id = _clutter_pixel_to_id (pixel);
+      actor = clutter_get_actor_by_gid (id);
+      goto result;
+    }
+
+  context->picks_per_frame++;
+
   _clutter_backend_ensure_context (context->backend, stage);
 
   /* needed for when a context switch happens */
   _clutter_stage_maybe_setup_viewport (stage);
 
-  if (G_LIKELY (!(clutter_pick_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
-    cogl_clip_push_window_rectangle (x, y, 1, 1);
+  /* If we are seeing multiple picks per frame that means the scene is static
+   * so we promote to doing a non-scissored pick render so that all subsequent
+   * picks for the same static scene won't require additional renders */
+  if (context->picks_per_frame < 2)
+    {
+      if (G_LIKELY (!(clutter_pick_debug_flags &
+                      CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+        cogl_clip_push_window_rectangle (x, y, 1, 1);
+      is_clipped = TRUE;
+    }
+  else
+    is_clipped = FALSE;
+
+  CLUTTER_NOTE (PICK, "Performing %s pick at %i,%i",
+                is_clipped ? "clippped" : "full", x, y);
 
   cogl_disable_fog ();
-  cogl_color_set_from_4ub (&stage_pick_id, 255, 255, 255, 255);
+  cogl_color_init_from_4ub (&stage_pick_id, 255, 255, 255, 255);
   CLUTTER_TIMER_START (_clutter_uprof_context, pick_clear);
   cogl_clear (&stage_pick_id,
 	      COGL_BUFFER_BIT_COLOR |
@@ -647,8 +673,14 @@ _clutter_do_pick (ClutterStage   *stage,
   context->pick_mode = CLUTTER_PICK_NONE;
   CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_paint);
 
-  if (G_LIKELY (!(clutter_pick_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
-    cogl_clip_pop ();
+  if (is_clipped)
+    {
+      if (G_LIKELY (!(clutter_pick_debug_flags &
+                      CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+        cogl_clip_pop ();
+    }
+  else
+    context->have_complete_pick_buffer = TRUE;
 
   /* Make sure Cogl flushes any batched geometry to the GPU driver */
   cogl_flush ();
@@ -761,12 +793,15 @@ static void
 update_pango_context (ClutterBackend *backend,
                       PangoContext   *context)
 {
+  ClutterSettings *settings;
   PangoFontDescription *font_desc;
   const cairo_font_options_t *font_options;
-  const gchar *font_name;
+  gchar *font_name;
   PangoDirection pango_dir;
   gdouble resolution;
 
+  settings = clutter_settings_get_default ();
+
   /* update the text direction */
   if (clutter_text_direction == CLUTTER_TEXT_DIRECTION_RTL)
     pango_dir = PANGO_DIRECTION_RTL;
@@ -775,8 +810,9 @@ update_pango_context (ClutterBackend *backend,
 
   pango_context_set_base_dir (context, pango_dir);
 
+  g_object_get (settings, "font-name", &font_name, NULL);
+
   /* get the configuration for the PangoContext from the backend */
-  font_name = clutter_backend_get_font_name (backend);
   font_options = clutter_backend_get_font_options (backend);
   resolution = clutter_backend_get_resolution (backend);
 
@@ -790,6 +826,7 @@ update_pango_context (ClutterBackend *backend,
   pango_cairo_context_set_resolution (context, resolution);
 
   pango_font_description_free (font_desc);
+  g_free (font_name);
 }
 
 PangoContext *
@@ -867,13 +904,13 @@ timed_poll (GPollFD *ufds,
   gint ret;
   CLUTTER_STATIC_TIMER (poll_timer,
                         "Mainloop", /* parent */
-                        "poll (idle)",
+                        "Mainloop Idle",
                         "The time spent idle in poll()",
                         0 /* no application private data */);
 
-  CLUTTER_TIMER_START (_clutter_uprof_context, poll_timer);
+  CLUTTER_TIMER_START (uprof_get_mainloop_context (), poll_timer);
   ret = prev_poll (ufds, nfsd, timeout_);
-  CLUTTER_TIMER_STOP (_clutter_uprof_context, poll_timer);
+  CLUTTER_TIMER_STOP (uprof_get_mainloop_context (), poll_timer);
   return ret;
 }
 #endif
@@ -894,7 +931,7 @@ clutter_main (void)
                         0 /* no application private data */);
 
   if (clutter_main_loop_level == 0)
-    CLUTTER_TIMER_START (_clutter_uprof_context, mainloop_timer);
+    CLUTTER_TIMER_START (uprof_get_mainloop_context (), mainloop_timer);
 
   /* Make sure there is a context */
   CLUTTER_CONTEXT ();
@@ -944,7 +981,7 @@ clutter_main (void)
   CLUTTER_MARK ();
 
   if (clutter_main_loop_level == 0)
-    CLUTTER_TIMER_STOP (_clutter_uprof_context, mainloop_timer);
+    CLUTTER_TIMER_STOP (uprof_get_mainloop_context (), mainloop_timer);
 }
 
 static void
@@ -995,7 +1032,7 @@ clutter_threads_init (void)
 }
 
 /**
- * clutter_threads_set_lock_functions:
+ * clutter_threads_set_lock_functions: (skip)
  * @enter_fn: function called when aquiring the Clutter main lock
  * @leave_fn: function called when releasing the Clutter main lock
  *
@@ -1163,6 +1200,8 @@ clutter_threads_dispatch_free (gpointer data)
  *                                  NULL);
  * ]|
  *
+ * Rename to: clutter_threads_add_idle
+ *
  * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 0.4
@@ -1188,7 +1227,7 @@ clutter_threads_add_idle_full (gint           priority,
 }
 
 /**
- * clutter_threads_add_idle:
+ * clutter_threads_add_idle: (skip)
  * @func: function to call
  * @data: data to pass to the function
  *
@@ -1235,6 +1274,8 @@ clutter_threads_add_idle (GSourceFunc func,
  *
  * See also clutter_threads_add_idle_full().
  *
+ * Rename to: clutter_threads_add_timeout
+ *
  * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 0.4
@@ -1262,7 +1303,7 @@ clutter_threads_add_timeout_full (gint           priority,
 }
 
 /**
- * clutter_threads_add_timeout:
+ * clutter_threads_add_timeout: (skip)
  * @interval: the time between calls to the function, in milliseconds
  * @func: function to call
  * @data: data to pass to the function
@@ -1313,6 +1354,8 @@ clutter_threads_add_timeout (guint       interval,
  *
  * See also clutter_threads_add_idle_full().
  *
+ * Rename to: clutter_threads_add_frame_source
+ *
  * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 0.8
@@ -1340,7 +1383,7 @@ clutter_threads_add_frame_source_full (gint           priority,
 }
 
 /**
- * clutter_threads_add_frame_source:
+ * clutter_threads_add_frame_source: (skip)
  * @fps: the number of times per second to call the function
  * @func: function to call
  * @data: data to pass to the function
@@ -1578,12 +1621,19 @@ clutter_init_real (GError **error)
     return CLUTTER_INIT_ERROR_BACKEND;
 
 #ifdef CLUTTER_ENABLE_PROFILE
-    {
-      UProfContext *cogl_context;
-      cogl_context = uprof_find_context ("Cogl");
-      if (cogl_context)
-        uprof_context_link (_clutter_uprof_context, cogl_context);
-    }
+  /* We need to be absolutely sure that uprof has been initialized
+   * before calling _clutter_uprof_init. uprof_init (NULL, NULL)
+   * will be a NOP if it has been initialized but it will also
+   * mean subsequent parsing of the UProf GOptionGroup will have no
+   * affect.
+   *
+   * Sadly GOptionGroup based library initialization is extremly
+   * fragile by design because GOptionGroups have no notion of
+   * dependencies and our post_parse_hook may be called before
+   * the cogl or uprof groups get parsed.
+   */
+  uprof_init (NULL, NULL);
+  _clutter_uprof_init ();
 
   if (clutter_profile_flags & CLUTTER_PROFILE_PICKING_ONLY)
     _clutter_profile_suspend ();
@@ -1794,7 +1844,7 @@ post_parse_hook (GOptionContext  *context,
 }
 
 /**
- * clutter_get_option_group:
+ * clutter_get_option_group: (skip)
  *
  * Returns a #GOptionGroup for the command line arguments recognized
  * by Clutter. You should add this group to your #GOptionContext with
@@ -1851,7 +1901,7 @@ clutter_get_option_group (void)
 }
 
 /**
- * clutter_get_option_group_without_init:
+ * clutter_get_option_group_without_init: (skip)
  *
  * Returns a #GOptionGroup for the command line arguments recognized
  * by Clutter. You should add this group to your #GOptionContext with
@@ -1952,6 +2002,14 @@ clutter_init_with_args (int            *argc,
       group = cogl_get_option_group ();
       g_option_context_add_group (context, group);
 
+      /* Note: That due to the implementation details of glib's goption
+       * parsing; cogl and uprof will not actually have there arguments
+       * parsed before the post_parse_hook is called! */
+#ifdef CLUTTER_ENABLE_PROFILE
+      group = uprof_get_option_group ();
+      g_option_context_add_group (context, group);
+#endif
+
       if (entries)
 	g_option_context_add_main_entries (context, entries, translation_domain);
 
@@ -1984,6 +2042,9 @@ clutter_parse_args (int    *argc,
 {
   GOptionContext *option_context;
   GOptionGroup   *clutter_group, *cogl_group;
+#ifdef CLUTTER_ENABLE_PROFILE
+  GOptionGroup   *uprof_group;
+#endif
   GError         *error = NULL;
   gboolean        ret = TRUE;
 
@@ -2002,6 +2063,11 @@ clutter_parse_args (int    *argc,
   cogl_group = cogl_get_option_group ();
   g_option_context_add_group (option_context, cogl_group);
 
+#ifdef CLUTTER_ENABLE_PROFILE
+  uprof_group = uprof_get_option_group ();
+  g_option_context_add_group (option_context, uprof_group);
+#endif
+
   if (!g_option_context_parse (option_context, argc, argv, &error))
     {
       if (error)
@@ -2094,13 +2160,18 @@ event_click_count_generate (ClutterEvent *event)
   static gint    previous_button_number = -1;
 
   ClutterInputDevice *device = NULL;
+  ClutterSettings *settings;
   ClutterBackend *backend;
   guint double_click_time;
   guint double_click_distance;
 
+  settings = clutter_settings_get_default ();
   backend = clutter_get_default_backend ();
-  double_click_distance = clutter_backend_get_double_click_distance (backend);
-  double_click_time = clutter_backend_get_double_click_time (backend);
+
+  g_object_get (settings,
+                "double-click-distance", &double_click_distance,
+                "double-click-time", &double_click_time,
+                NULL);
 
   device = clutter_event_get_device (event);
   if (device != NULL)
@@ -2543,7 +2614,7 @@ _clutter_process_event (ClutterEvent *event)
 
 
 /**
- * clutter_get_actor_by_gid
+ * clutter_get_actor_by_gid:
  * @id: a #ClutterActor ID.
  *
  * Retrieves the #ClutterActor with @id.
