@@ -81,6 +81,8 @@ struct _ClutterTexturePrivate
 
   CoglHandle pick_material;
 
+  gchar *filename;
+
   ClutterTextureAsyncData *async_data;
 
   guint no_slice : 1;
@@ -142,9 +144,13 @@ enum
   PROP_KEEP_ASPECT_RATIO,
   PROP_LOAD_ASYNC,
   PROP_LOAD_DATA_ASYNC,
-  PROP_PICK_WITH_ALPHA
+  PROP_PICK_WITH_ALPHA,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   SIZE_CHANGE,
@@ -161,6 +167,8 @@ static guint        repaint_upload_func = 0;
 static GList       *upload_list = NULL;
 static GStaticMutex upload_list_mutex = G_STATIC_MUTEX_INIT;
 
+static CoglMaterial *texture_template_material = NULL;
+
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
 
@@ -209,10 +217,12 @@ texture_free_gl_resources (ClutterTexture *texture)
   CLUTTER_MARK();
 
   if (priv->material != COGL_INVALID_HANDLE)
-    /* We want to keep the layer so that the filter settings will
-       remain but we want to free its resources so we clear the
-       texture handle */
-    cogl_material_set_layer (priv->material, 0, COGL_INVALID_HANDLE);
+    {
+      /* We want to keep the layer so that the filter settings will
+         remain but we want to free its resources so we clear the
+         texture handle */
+      cogl_material_set_layer (priv->material, 0, COGL_INVALID_HANDLE);
+    }
 }
 
 static void
@@ -499,13 +509,27 @@ update_fbo (ClutterActor *self)
 
       /* Reapply the source's parent transformations */
       if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
-        _clutter_actor_apply_modelview_transform_recursive (source_parent,
-                                                            NULL);
+        {
+          CoglMatrix modelview;
+
+          /* NB: _clutter_actor_apply_modelview_transform_recursive
+           * will never include the transformation between stage
+           * coordinates and OpenGL window coordinates, we have to
+           * explicitly use the stage->apply_transform to get that...
+           */
+
+          cogl_matrix_init_identity (&modelview);
+          _clutter_actor_apply_modelview_transform (stage, &modelview);
+          _clutter_actor_apply_modelview_transform_recursive (source_parent,
+                                                              NULL,
+                                                              &modelview);
+          cogl_set_modelview_matrix (&modelview);
+        }
     }
 
 
   /* cogl_clear is called to clear the buffers */
-  cogl_color_set_from_4ub (&transparent_col, 0, 0, 0, 0);
+  cogl_color_init_from_4ub (&transparent_col, 0, 0, 0, 0);
   cogl_clear (&transparent_col,
               COGL_BUFFER_BIT_COLOR |
               COGL_BUFFER_BIT_DEPTH);
@@ -553,7 +577,7 @@ create_pick_material (ClutterActor *self)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (self);
   ClutterTexturePrivate *priv = texture->priv;
-  CoglHandle pick_material = cogl_material_new ();
+  CoglHandle pick_material = cogl_material_copy (texture_template_material);
   GError *error = NULL;
 
   if (!cogl_material_set_layer_combine (pick_material, 0,
@@ -609,11 +633,11 @@ clutter_texture_pick (ClutterActor       *self,
       if (priv->fbo_handle != COGL_INVALID_HANDLE)
         update_fbo (self);
 
-      cogl_color_set_from_4ub (&pick_color,
-                               color->red,
-                               color->green,
-                               color->blue,
-                               0xff);
+      cogl_color_init_from_4ub (&pick_color,
+                                color->red,
+                                color->green,
+                                color->blue,
+                                0xff);
       cogl_material_set_layer_combine_constant (priv->pick_material,
                                                 0, &pick_color);
       cogl_material_set_layer (priv->pick_material, 0,
@@ -726,21 +750,13 @@ static void
 clutter_texture_dispose (GObject *object)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (object);
+  ClutterTexturePrivate *priv = texture->priv;
 
   texture_free_gl_resources (texture);
   texture_fbo_free_resources (texture);
 
   clutter_texture_async_load_cancel (texture);
 
-  G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
-}
-
-static void
-clutter_texture_finalize (GObject *object)
-{
-  ClutterTexture *texture = CLUTTER_TEXTURE (object);
-  ClutterTexturePrivate *priv = texture->priv;
-
   if (priv->material != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->material);
@@ -753,6 +769,16 @@ clutter_texture_finalize (GObject *object)
       priv->pick_material = COGL_INVALID_HANDLE;
     }
 
+  G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
+}
+
+static void
+clutter_texture_finalize (GObject *object)
+{
+  ClutterTexturePrivate *priv = CLUTTER_TEXTURE (object)->priv;
+
+  g_free (priv->filename);
+
   G_OBJECT_CLASS (clutter_texture_parent_class)->finalize (object);
 }
 
@@ -900,6 +926,10 @@ clutter_texture_get_property (GObject    *object,
       g_value_set_boolean (value, priv->pick_with_alpha);
       break;
 
+    case PROP_FILENAME:
+      g_value_set_string (value, priv->filename);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -911,6 +941,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterTexturePrivate));
 
@@ -928,105 +959,106 @@ clutter_texture_class_init (ClutterTextureClass *klass)
   gobject_class->set_property = clutter_texture_set_property;
   gobject_class->get_property = clutter_texture_get_property;
 
-  g_object_class_install_property
-    (gobject_class, PROP_SYNC_SIZE,
-     g_param_spec_boolean ("sync-size",
-			   "Sync size of actor",
-			   "Auto sync size of actor to underlying pixbuf "
-			   "dimensions",
-			   TRUE,
-			   CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("sync-size",
+                                P_("Sync size of actor"),
+                                P_("Auto sync size of actor to underlying pixbuf dimensions"),
+                                TRUE,
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_SYNC_SIZE] = pspec;
+  g_object_class_install_property (gobject_class, PROP_SYNC_SIZE, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_NO_SLICE,
-     g_param_spec_boolean ("disable-slicing",
-			   "Disable Slicing",
-			   "Force the underlying texture to be singlular"
-			   "and not made of of smaller space saving "
-                           "inidivual textures.",
-			   FALSE,
-			   G_PARAM_CONSTRUCT_ONLY | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("disable-slicing",
+                                P_("Disable Slicing"),
+                                P_("Forces the underlying texture to be singlular and not made of of smaller space saving inidivual textures."),
+                                FALSE,
+                                G_PARAM_CONSTRUCT_ONLY |
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_NO_SLICE] = pspec;
+  g_object_class_install_property (gobject_class, PROP_NO_SLICE, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_MAX_TILE_WASTE,
-     g_param_spec_int ("tile-waste",
-                       "Tile Waste",
-                       "Maximum waste area of a sliced texture",
-                       -1, G_MAXINT,
-                       COGL_TEXTURE_MAX_WASTE,
-                       CLUTTER_PARAM_READABLE));
+  pspec = g_param_spec_int ("tile-waste",
+                            P_("Tile Waste"),
+                            P_("Maximum waste area of a sliced texture"),
+                            -1, G_MAXINT,
+                            COGL_TEXTURE_MAX_WASTE,
+                            CLUTTER_PARAM_READABLE);
+  obj_props[PROP_MAX_TILE_WASTE] = pspec;
+  g_object_class_install_property (gobject_class, PROP_MAX_TILE_WASTE, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_REPEAT_X,
-     g_param_spec_boolean ("repeat-x",
-			   "Tile underlying pixbuf in x direction",
-			   "Repeat underlying pixbuf rather than scale "
-			   "in x direction.",
-			   FALSE,
-			   CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("repeat-x",
+                                P_("Horizontal repeat"),
+                                P_("Repeat the contents rather than scaling them horizontally."),
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_REPEAT_X] = pspec;
+  g_object_class_install_property (gobject_class, PROP_REPEAT_X, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_REPEAT_Y,
-     g_param_spec_boolean ("repeat-y",
-			   "Tile underlying pixbuf in y direction",
-			   "Repeat underlying pixbuf rather than scale "
-			   "in y direction.",
-			   FALSE,
-			   CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("repeat-y",
+                                P_("Vertical repeat"),
+                                P_("Repeat the contents rather than scaling them vertically."),
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_REPEAT_Y] = pspec;
+  g_object_class_install_property (gobject_class, PROP_REPEAT_Y, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_FILTER_QUALITY,
-     g_param_spec_enum ("filter-quality",
-                       "Filter Quality",
-                       "Rendering quality used when drawing the texture.",
-                       CLUTTER_TYPE_TEXTURE_QUALITY,
-		       CLUTTER_TEXTURE_QUALITY_MEDIUM,
-		       G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_enum ("filter-quality",
+                             P_("Filter Quality"),
+                             P_("Rendering quality used when drawing the texture."),
+                             CLUTTER_TYPE_TEXTURE_QUALITY,
+                             CLUTTER_TEXTURE_QUALITY_MEDIUM,
+                             G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_FILTER_QUALITY] = pspec;
+  g_object_class_install_property (gobject_class, PROP_FILTER_QUALITY, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_PIXEL_FORMAT,
-     g_param_spec_enum ("pixel-format",
-		        "Texture pixel format",
-		        "CoglPixelFormat to use.",
-                        COGL_TYPE_PIXEL_FORMAT,
-		        COGL_PIXEL_FORMAT_RGBA_8888,
-		        CLUTTER_PARAM_READABLE));
+  pspec = g_param_spec_enum ("pixel-format",
+                             P_("Pixel Format"),
+                             P_("The Cogl pixel format to use."),
+                             COGL_TYPE_PIXEL_FORMAT,
+                             COGL_PIXEL_FORMAT_RGBA_8888,
+                             CLUTTER_PARAM_READABLE);
+  obj_props[PROP_PIXEL_FORMAT] = pspec;
+  g_object_class_install_property (gobject_class, PROP_PIXEL_FORMAT, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_COGL_TEXTURE,
-     g_param_spec_boxed ("cogl-texture",
-			 "COGL Texture",
-			 "The underlying COGL texture handle used to draw "
-			 "this actor",
-			 COGL_TYPE_HANDLE,
-			 G_PARAM_READWRITE));
+  pspec = g_param_spec_boxed ("cogl-texture",
+                              P_("COGL Texture"),
+                              P_("The underlying COGL texture handle used to draw this actor"),
+                              COGL_TYPE_HANDLE,
+                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_COGL_TEXTURE] = pspec;
+  g_object_class_install_property (gobject_class, PROP_COGL_TEXTURE, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_COGL_MATERIAL,
-     g_param_spec_boxed ("cogl-material",
-			 "COGL Material",
-			 "The underlying COGL material handle used to draw "
-			 "this actor",
-			 COGL_TYPE_HANDLE,
-			 G_PARAM_READWRITE));
+  pspec = g_param_spec_boxed ("cogl-material",
+                              P_("COGL Material"),
+                              P_("The underlying COGL material handle used to draw this actor"),
+                              COGL_TYPE_HANDLE,
+                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_COGL_MATERIAL] = pspec;
+  g_object_class_install_property (gobject_class, PROP_COGL_MATERIAL, pspec);
 
-  g_object_class_install_property
-    (gobject_class, PROP_FILENAME,
-     g_param_spec_string ("filename",
-                          "Filename",
-                          "The full path of the file containing the texture",
-                          NULL,
-                          G_PARAM_WRITABLE));
-
-  g_object_class_install_property
-    (gobject_class, PROP_KEEP_ASPECT_RATIO,
-     g_param_spec_boolean ("keep-aspect-ratio",
-			   "Keep Aspect Ratio",
-			   "Keep the aspect ratio of the texture when "
-			   "requesting the preferred width or height",
-			   FALSE,
-			   CLUTTER_PARAM_READWRITE));
+  /**
+   * ClutterTexture:filename:
+   *
+   * The path of the file containing the image data to be displayed by
+   * the texture.
+   *
+   * This property is unset when using the clutter_texture_set_from_*_data()
+   * family of functions.
+   */
+  pspec = g_param_spec_string ("filename",
+                               P_("Filename"),
+                               P_("The path of the file containing the image data"),
+                               NULL,
+                               CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_FILENAME] = pspec;
+  g_object_class_install_property (gobject_class, PROP_FILENAME, pspec);
 
+  pspec = g_param_spec_boolean ("keep-aspect-ratio",
+                                P_("Keep Aspect Ratio"),
+                                P_("Keep the aspect ratio of the texture when requesting the preferred width or height"),
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_KEEP_ASPECT_RATIO] = pspec;
+  g_object_class_install_property (gobject_class, PROP_KEEP_ASPECT_RATIO, pspec);
 
   /**
    * ClutterTexture:load-async:
@@ -1047,14 +1079,13 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    *
    * Since: 1.0
    */
-  g_object_class_install_property
-    (gobject_class, PROP_LOAD_ASYNC,
-     g_param_spec_boolean ("load-async",
-			   "Load asynchronously",
-			   "Load files inside a thread to avoid blocking when "
-                           "loading images.",
-			   FALSE,
-			   CLUTTER_PARAM_WRITABLE));
+  pspec = g_param_spec_boolean ("load-async",
+                                P_("Load asynchronously"),
+                                P_("Load files inside a thread to avoid blocking when loading images from disk."),
+                                FALSE,
+                                CLUTTER_PARAM_WRITABLE);
+  obj_props[PROP_LOAD_ASYNC] = pspec;
+  g_object_class_install_property (gobject_class, PROP_LOAD_ASYNC, pspec);
 
 
   /**
@@ -1065,14 +1096,13 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    *
    * Since: 1.0
    */
-  g_object_class_install_property
-    (gobject_class, PROP_LOAD_DATA_ASYNC,
-     g_param_spec_boolean ("load-data-async",
-			   "Load data asynchronously",
-			   "Decode image data files inside a thread to reduce "
-                           "blocking when loading images.",
-			   FALSE,
-			   CLUTTER_PARAM_WRITABLE));
+  pspec = g_param_spec_boolean ("load-data-async",
+                                P_("Load data asynchronously"),
+                                P_("Decode image data files inside a thread to reduce blocking when loading images from disk."),
+                                FALSE,
+                                CLUTTER_PARAM_WRITABLE);
+  obj_props[PROP_LOAD_DATA_ASYNC] = pspec;
+  g_object_class_install_property (gobject_class, PROP_LOAD_DATA_ASYNC, pspec);
 
   /**
    * ClutterTexture::pick-with-alpha:
@@ -1091,13 +1121,13 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    *
    * Since: 1.4
    */
-  g_object_class_install_property
-    (gobject_class, PROP_PICK_WITH_ALPHA,
-     g_param_spec_boolean ("pick-with-alpha",
-			   "Pick With Alpha Channel",
-                           "Shape actor with alpha channel when picking",
-			   FALSE,
-			   CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("pick-with-alpha",
+                                P_("Pick With Alpha"),
+                                P_("Shape actor with alpha channel when picking"),
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_PICK_WITH_ALPHA] = pspec;
+  g_object_class_install_property (gobject_class, PROP_PICK_WITH_ALPHA, pspec);
 
   /**
    * ClutterTexture::size-change:
@@ -1222,13 +1252,26 @@ clutter_texture_init (ClutterTexture *self)
   priv->repeat_x          = FALSE;
   priv->repeat_y          = FALSE;
   priv->sync_actor_size   = TRUE;
-  priv->material          = cogl_material_new ();
   priv->fbo_handle        = COGL_INVALID_HANDLE;
   priv->pick_material     = COGL_INVALID_HANDLE;
   priv->keep_aspect_ratio = FALSE;
   priv->pick_with_alpha   = FALSE;
   priv->pick_with_alpha_supported = TRUE;
   priv->seen_create_pick_material_warning = FALSE;
+
+  if (G_UNLIKELY (texture_template_material == NULL))
+    {
+      CoglHandle dummy_tex;
+
+      dummy_tex = cogl_texture_new_with_size (1, 1, COGL_TEXTURE_NO_SLICING,
+                                              COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+
+      texture_template_material = cogl_material_new ();
+      cogl_material_set_layer (texture_template_material, 0, dummy_tex);
+      cogl_handle_unref (dummy_tex);
+    }
+
+  priv->material = cogl_material_copy (texture_template_material);
 }
 
 /**
@@ -1236,12 +1279,13 @@ clutter_texture_init (ClutterTexture *self)
  * @texture: A #ClutterTexture
  *
  * Returns a handle to the underlying COGL material used for drawing
- * the actor. No extra reference is taken so if you need to keep the
- * handle then you should call cogl_handle_ref() on it.
+ * the actor.
+ *
+ * Return value: (transfer none): a handle for a #CoglMaterial. The
+ *   material is owned by the #ClutterTexture and it should not be
+ *   unreferenced
  *
  * Since: 1.0
- *
- * Return value: COGL material handle
  */
 CoglHandle
 clutter_texture_get_cogl_material (ClutterTexture *texture)
@@ -1278,7 +1322,6 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 
   cogl_handle_ref (cogl_material);
 
-  /* This */
   if (texture->priv->material)
     cogl_handle_unref (texture->priv->material);
 
@@ -1293,7 +1336,7 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 }
 
 /**
- * clutter_texture_get_cogl_texture
+ * clutter_texture_get_cogl_texture:
  * @texture: A #ClutterTexture
  *
  * Retrieves the handle to the underlying COGL texture used for drawing
@@ -1305,23 +1348,32 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
  * layers you should use clutter_texture_get_cogl_material() instead
  * and use the #CoglMaterial API.
  *
- * Since: 0.8
+ * Return value: (transfer none): a #CoglHandle for the texture. The returned
+ *   handle is owned by the #ClutterTexture and it should not be unreferenced
  *
- * Return value: COGL texture handle
+ * Since: 0.8
  */
 CoglHandle
 clutter_texture_get_cogl_texture (ClutterTexture *texture)
 {
+  CoglMaterialLayerType layer_type;
   const GList *layers;
   int n_layers;
 
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), COGL_INVALID_HANDLE);
 
   layers = cogl_material_get_layers (texture->priv->material);
+  if (layers == NULL)
+    return COGL_INVALID_HANDLE;
+
   n_layers = g_list_length ((GList *)layers);
   if (n_layers == 0)
     return COGL_INVALID_HANDLE;
 
+  layer_type = cogl_material_layer_get_type (layers->data);
+  if (layer_type != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
+    return COGL_INVALID_HANDLE;
+
   return cogl_material_layer_get_texture (layers->data);
 }
 
@@ -1390,16 +1442,39 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
                      priv->image_height);
 
       if (priv->sync_actor_size)
-        clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
+        {
+          ClutterActor *actor = CLUTTER_ACTOR (texture);
+
+          /* we have been requested to keep the actor size in
+           * sync with the texture data; if we also want to
+           * maintain the aspect ratio we want to change the
+           * requisition mode depending on the orientation of
+           * the texture, so that the parent container can do
+           * the right thing
+           */
+          if (priv->keep_aspect_ratio)
+            {
+              ClutterRequestMode request;
+
+              if (priv->image_width >= priv->image_height)
+                request = CLUTTER_REQUEST_HEIGHT_FOR_WIDTH;
+              else
+                request = CLUTTER_REQUEST_WIDTH_FOR_HEIGHT;
+
+              clutter_actor_set_request_mode (actor, request);
+            }
+
+          clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
+        }
     }
 
   /* rename signal */
   g_signal_emit (texture, texture_signals[PIXBUF_CHANGE], 0);
 
-  g_object_notify (G_OBJECT (texture), "cogl-texture");
-
   /* If resized actor may need resizing but paint() will do this */
   clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+
+  _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_COGL_TEXTURE]);
 }
 
 static gboolean
@@ -1447,6 +1522,9 @@ clutter_texture_set_from_data (ClutterTexture     *texture,
       return FALSE;
     }
 
+  g_free (priv->filename);
+  priv->filename = NULL;
+
   clutter_texture_set_cogl_texture (texture, new_texture);
 
   cogl_handle_unref (new_texture);
@@ -1936,12 +2014,17 @@ clutter_texture_set_from_file (ClutterTexture *texture,
       return FALSE;
     }
 
+  g_free (priv->filename);
+  priv->filename = g_strdup (filename);
+
   clutter_texture_set_cogl_texture (texture, new_texture);
 
   cogl_handle_unref (new_texture);
 
   g_signal_emit (texture, texture_signals[LOAD_FINISHED], 0, NULL);
 
+  _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_FILENAME]);
+
   return TRUE;
 }
 
@@ -1986,9 +2069,9 @@ clutter_texture_set_filter_quality (ClutterTexture        *texture,
       cogl_material_set_layer_filters (priv->material, 0,
                                        min_filter, mag_filter);
 
-      g_object_notify (G_OBJECT (texture), "filter-quality");
-
       clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_FILTER_QUALITY]);
     }
 }
 
@@ -2229,6 +2312,9 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
       return FALSE;
     }
 
+  g_free (texture->priv->filename);
+  texture->priv->filename = NULL;
+
   /* rename signal */
   g_signal_emit (texture, texture_signals[PIXBUF_CHANGE], 0);
 
@@ -2530,7 +2616,7 @@ clutter_texture_set_sync_size (ClutterTexture *texture,
 
       clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
 
-      g_object_notify (G_OBJECT (texture), "sync-size");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_SYNC_SIZE]);
     }
 }
 
@@ -2538,7 +2624,7 @@ clutter_texture_set_sync_size (ClutterTexture *texture,
  * clutter_texture_get_sync_size:
  * @texture: a #ClutterTexture
  *
- * Retrieves the value set with clutter_texture_get_sync_size()
+ * Retrieves the value set with clutter_texture_set_sync_size()
  *
  * Return value: %TRUE if the #ClutterTexture should have the same
  *   preferred size of the underlying image data
@@ -2582,7 +2668,7 @@ clutter_texture_set_repeat (ClutterTexture *texture,
     {
       priv->repeat_x = repeat_x;
 
-      g_object_notify (G_OBJECT (texture), "repeat-x");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_REPEAT_X]);
 
       changed = TRUE;
     }
@@ -2591,7 +2677,7 @@ clutter_texture_set_repeat (ClutterTexture *texture,
     {
       priv->repeat_y = repeat_y;
 
-      g_object_notify (G_OBJECT (texture), "repeat-y");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_REPEAT_Y]);
 
       changed = TRUE;
     }
@@ -2685,7 +2771,7 @@ clutter_texture_set_keep_aspect_ratio (ClutterTexture *texture,
 
       clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
 
-      g_object_notify (G_OBJECT (texture), "keep-aspect-ratio");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_KEEP_ASPECT_RATIO]);
     }
 }
 
@@ -2693,7 +2779,7 @@ clutter_texture_set_keep_aspect_ratio (ClutterTexture *texture,
  * clutter_texture_get_keep_aspect_ratio:
  * @texture: a #ClutterTexture
  *
- * Retrieves the value set using clutter_texture_get_keep_aspect_ratio()
+ * Retrieves the value set using clutter_texture_set_keep_aspect_ratio()
  *
  * Return value: %TRUE if the #ClutterTexture should maintain the
  *   aspect ratio of the underlying image
@@ -2740,8 +2826,8 @@ clutter_texture_set_load_async (ClutterTexture *texture,
 
       priv->load_async_set = load_async;
 
-      g_object_notify (G_OBJECT (texture), "load-async");
-      g_object_notify (G_OBJECT (texture), "load-data-async");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_LOAD_ASYNC]);
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_LOAD_DATA_ASYNC]);
     }
 }
 
@@ -2749,7 +2835,7 @@ clutter_texture_set_load_async (ClutterTexture *texture,
  * clutter_texture_get_load_async:
  * @texture: a #ClutterTexture
  *
- * Retrieves the value set using clutter_texture_get_load_async()
+ * Retrieves the value set using clutter_texture_set_load_async()
  *
  * Return value: %TRUE if the #ClutterTexture should load the data from
  *   disk asynchronously
@@ -2798,8 +2884,8 @@ clutter_texture_set_load_data_async (ClutterTexture *texture,
 
       priv->load_async_set = load_async;
 
-      g_object_notify (G_OBJECT (texture), "load-async");
-      g_object_notify (G_OBJECT (texture), "load-data-async");
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_LOAD_ASYNC]);
+      _clutter_notify_by_pspec (G_OBJECT (texture), obj_props[PROP_LOAD_DATA_ASYNC]);
     }
 }
 
@@ -2864,6 +2950,11 @@ clutter_texture_set_pick_with_alpha (ClutterTexture *texture,
 
   /* NB: the pick material is created lazily when we first pick */
   priv->pick_with_alpha = pick_with_alpha;
+
+  /* NB: actors are expected to call clutter_actor_queue_redraw when
+   * ever some state changes that will affect painting *or picking...
+   */
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 }
 
 /**
