@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -85,9 +85,13 @@ enum
   PROP_LOOP,
   PROP_DELAY,
   PROP_DURATION,
-  PROP_DIRECTION
+  PROP_DIRECTION,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   NEW_FRAME,
@@ -259,6 +263,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                                 "Should the timeline automatically restart",
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_LOOP] = pspec;
   g_object_class_install_property (object_class, PROP_LOOP, pspec);
 
   /**
@@ -275,6 +280,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                              0, G_MAXUINT,
                              0,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DELAY] = pspec;
   g_object_class_install_property (object_class, PROP_DELAY, pspec);
 
   /**
@@ -291,6 +297,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                              0, G_MAXUINT,
                              1000,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (object_class, PROP_DURATION, pspec);
 
   /**
@@ -307,6 +314,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                              CLUTTER_TYPE_TIMELINE_DIRECTION,
                              CLUTTER_TIMELINE_FORWARD,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DIRECTION] = pspec;
   g_object_class_install_property (object_class, PROP_DIRECTION, pspec);
 
   /**
@@ -324,7 +332,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, new_frame),
 		  NULL, NULL,
-		  clutter_marshal_VOID__INT,
+		  _clutter_marshal_VOID__INT,
 		  G_TYPE_NONE,
 		  1, G_TYPE_INT);
   /**
@@ -340,7 +348,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, completed),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterTimeline::started:
@@ -357,7 +365,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, started),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterTimeline::paused:
@@ -371,7 +379,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, paused),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterTimeline::marker-reached:
@@ -411,7 +419,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                   G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
                   G_STRUCT_OFFSET (ClutterTimelineClass, marker_reached),
                   NULL, NULL,
-                  clutter_marshal_VOID__STRING_INT,
+                  _clutter_marshal_VOID__STRING_INT,
                   G_TYPE_NONE, 2,
                   G_TYPE_STRING,
                   G_TYPE_INT);
@@ -812,7 +820,7 @@ clutter_timeline_set_loop (ClutterTimeline *timeline,
     {
       timeline->priv->loop = loop;
 
-      g_object_notify (G_OBJECT (timeline), "loop");
+      _clutter_notify_by_pspec (G_OBJECT (timeline), obj_props[PROP_LOOP]);
     }
 }
 
@@ -957,7 +965,8 @@ clutter_timeline_is_playing (ClutterTimeline *timeline)
  * be started and will not be positioned to the current position of
  * @timeline: you will have to start it with clutter_timeline_start().
  *
- * Return Value: a new #ClutterTimeline, cloned from @timeline
+ * Return Value: (transfer full): a new #ClutterTimeline, cloned
+ *   from @timeline
  *
  * Since: 0.4
  */
@@ -1037,7 +1046,7 @@ clutter_timeline_set_delay (ClutterTimeline *timeline,
   if (priv->delay != msecs)
     {
       priv->delay = msecs;
-      g_object_notify (G_OBJECT (timeline), "delay");
+      _clutter_notify_by_pspec (G_OBJECT (timeline), obj_props[PROP_DELAY]);
     }
 }
 
@@ -1089,7 +1098,7 @@ clutter_timeline_set_duration (ClutterTimeline *timeline,
     {
       priv->duration = msecs;
 
-      g_object_notify (G_OBJECT (timeline), "duration");
+      _clutter_notify_by_pspec (G_OBJECT (timeline), obj_props[PROP_DURATION]);
     }
 }
 
@@ -1162,7 +1171,7 @@ clutter_timeline_set_direction (ClutterTimeline          *timeline,
       if (priv->elapsed_time == 0)
         priv->elapsed_time = priv->duration;
 
-      g_object_notify (G_OBJECT (timeline), "direction");
+      _clutter_notify_by_pspec (G_OBJECT (timeline), obj_props[PROP_DIRECTION]);
     }
 }
 
@@ -1331,9 +1340,9 @@ collect_markers (const gchar *key,
  * negative integer, all the markers attached to @timeline will be
  * returned.
  *
- * Return value: (array zero-terminated=1 length=n_markers): a newly
- *   allocated, %NULL terminated string array containing the names of
- *   the markers. Use g_strfreev() when done.
+ * Return value: (transfer full) (array zero-terminated=1 length=n_markers):
+ *   a newly allocated, %NULL terminated string array containing the names
+ *   of the markers. Use g_strfreev() when done.
  *
  * Since: 0.8
  */
