@@ -40,10 +40,19 @@ G_BEGIN_DECLS
 
 /* Forward delarations to avoid header catch 22's */
 typedef struct _ClutterActor            ClutterActor;
+
 typedef struct _ClutterStage            ClutterStage;
 typedef struct _ClutterContainer        ClutterContainer; /* dummy */
 typedef struct _ClutterChildMeta        ClutterChildMeta;
 typedef struct _ClutterLayoutMeta       ClutterLayoutMeta;
+typedef struct _ClutterActorMeta        ClutterActorMeta;
+
+typedef struct _ClutterAnimator         ClutterAnimator;
+
+typedef struct _ClutterAction           ClutterAction;
+typedef struct _ClutterConstraint       ClutterConstraint;
+
+typedef union _ClutterEvent             ClutterEvent;
 
 /**
  * ClutterGravity:
@@ -179,6 +188,12 @@ struct _ClutterGeometry
 
 GType clutter_geometry_get_type (void) G_GNUC_CONST;
 
+void      clutter_geometry_union      (const ClutterGeometry *geometry_a,
+                                       const ClutterGeometry *geometry_b,
+                                       ClutterGeometry       *result);
+gboolean  clutter_geometry_intersects (const ClutterGeometry *geometry0,
+                                       const ClutterGeometry *geometry1);
+
 /**
  * ClutterKnot:
  * @x: X coordinate of the knot
@@ -384,7 +399,7 @@ typedef enum {
 typedef enum
 {
   CLUTTER_FONT_MIPMAPPING = (1 << 0),
-  CLUTTER_FONT_HINTING    = (1 << 1),
+  CLUTTER_FONT_HINTING    = (1 << 1)
 } ClutterFontFlags;
 
 /**
@@ -404,6 +419,20 @@ typedef enum {
   CLUTTER_TEXT_DIRECTION_RTL
 } ClutterTextDirection;
 
+/**
+ * ClutterShaderType:
+ * @CLUTTER_VERTEX_SHADER: a vertex shader
+ * @CLUTTER_FRAGMENT_SHADER: a fragment shader
+ *
+ * The type of GLSL shader program
+ *
+ * Since: 1.4
+ */
+typedef enum {
+  CLUTTER_VERTEX_SHADER,
+  CLUTTER_FRAGMENT_SHADER
+} ClutterShaderType;
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TYPES_H__ */
