@@ -36,7 +36,7 @@
 #include "cogl-bitmap.h"
 #include "cogl-bitmap-private.h"
 #include "cogl-buffer-private.h"
-#include "cogl-pixel-buffer-private.h"
+#include "cogl-pixel-array-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-texture-2d-sliced-private.h"
@@ -152,21 +152,19 @@ _cogl_texture_determine_internal_format (CoglPixelFormat src_format,
     return dst_format;
 }
 
-gboolean
+CoglBitmap *
 _cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
                                   CoglPixelFormat  dst_format,
                                   CoglPixelFormat *dst_format_out,
-                                  CoglBitmap      *dst_bmp,
-                                  gboolean        *copied_bitmap,
                                   GLenum          *out_glintformat,
                                   GLenum          *out_glformat,
                                   GLenum          *out_gltype)
 {
-  dst_format = _cogl_texture_determine_internal_format (src_bmp->format,
-                                                        dst_format);
+  CoglPixelFormat src_format = _cogl_bitmap_get_format (src_bmp);
+  CoglBitmap *dst_bmp;
 
-  *copied_bitmap = FALSE;
-  *dst_bmp = *src_bmp;
+  dst_format = _cogl_texture_determine_internal_format (src_format,
+                                                        dst_format);
 
   /* OpenGL supports specifying a different format for the internal
      format when uploading texture data. We should use this to convert
@@ -180,26 +178,46 @@ _cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
 
   /* If the source format does not have the same premult flag as the
      dst format then we need to copy and convert it */
-  if (_cogl_texture_needs_premult_conversion (src_bmp->format,
+  if (_cogl_texture_needs_premult_conversion (src_format,
                                               dst_format))
     {
-      dst_bmp->data = g_memdup (dst_bmp->data,
-                                dst_bmp->height * dst_bmp->rowstride);
-      *copied_bitmap = TRUE;
+      guint8 *src_data;
+      guint8 *dst_data;
+
+      if ((src_data = _cogl_bitmap_map (src_bmp,
+                                        COGL_BUFFER_ACCESS_READ, 0)) == NULL)
+        return NULL;
+
+      dst_data = g_memdup (src_data,
+                           _cogl_bitmap_get_height (src_bmp) *
+                           _cogl_bitmap_get_rowstride (src_bmp));
+
+      _cogl_bitmap_unmap (src_bmp);
+
+      dst_bmp =
+        _cogl_bitmap_new_from_data (dst_data,
+                                    src_format,
+                                    _cogl_bitmap_get_width (src_bmp),
+                                    _cogl_bitmap_get_height (src_bmp),
+                                    _cogl_bitmap_get_rowstride (src_bmp),
+                                    (CoglBitmapDestroyNotify) g_free,
+                                    NULL);
 
       if (!_cogl_bitmap_convert_premult_status (dst_bmp,
-                                                src_bmp->format ^
+                                                src_format ^
                                                 COGL_PREMULT_BIT))
         {
-          g_free (dst_bmp->data);
-          return FALSE;
+          cogl_object_unref (dst_bmp);
+          return NULL;
         }
     }
+  else
+    dst_bmp = cogl_object_ref (src_bmp);
 
   /* Use the source format from the src bitmap type and the internal
      format from the dst format type so that GL can do the
      conversion */
-  _cogl_pixel_format_to_gl (src_bmp->format,
+  _cogl_pixel_format_to_gl (src_format,
                             NULL, /* internal format */
                             out_glformat,
                             out_gltype);
@@ -212,28 +230,23 @@ _cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
   {
     CoglPixelFormat closest_format;
 
-    closest_format = _cogl_pixel_format_to_gl (dst_bmp->format,
+    closest_format = _cogl_pixel_format_to_gl (dst_format,
                                                out_glintformat,
                                                out_glformat,
                                                out_gltype);
 
-
-    if (closest_format != src_bmp->format)
-      {
-        if (!_cogl_bitmap_convert_format_and_premult (src_bmp,
-                                                      dst_bmp,
-                                                      closest_format))
-          return FALSE;
-
-        *copied_bitmap = TRUE;
-      }
+    if (closest_format != src_format)
+      dst_bmp = _cogl_bitmap_convert_format_and_premult (src_bmp,
+                                                         closest_format);
+    else
+      dst_bmp = cogl_object_ref (src_bmp);
   }
 #endif /* HAVE_COGL_GL */
 
   if (dst_format_out)
     *dst_format_out = dst_format;
 
-  return TRUE;
+  return dst_bmp;
 }
 
 void
@@ -267,14 +280,14 @@ void
 _cogl_texture_set_wrap_mode_parameters (CoglHandle handle,
                                         GLenum wrap_mode_s,
                                         GLenum wrap_mode_t,
-                                        GLenum wrap_mode_r)
+                                        GLenum wrap_mode_p)
 {
   CoglTexture *tex = COGL_TEXTURE (handle);
 
   tex->vtable->set_wrap_mode_parameters (tex,
                                          wrap_mode_s,
                                          wrap_mode_t,
-                                         wrap_mode_r);
+                                         wrap_mode_p);
 }
 
 /* This is like CoglSpanIter except it deals with floats and it
@@ -402,7 +415,8 @@ cogl_texture_new_from_data (unsigned int      width,
 			    unsigned int      rowstride,
 			    const guint8     *data)
 {
-  CoglBitmap bitmap;
+  CoglBitmap *bmp;
+  CoglHandle tex;
 
   if (format == COGL_PIXEL_FORMAT_ANY)
     return COGL_INVALID_HANDLE;
@@ -415,13 +429,18 @@ cogl_texture_new_from_data (unsigned int      width,
     rowstride = width * _cogl_get_format_bpp (format);
 
   /* Wrap the data into a bitmap */
-  bitmap.width = width;
-  bitmap.height = height;
-  bitmap.data = (guint8 *) data;
-  bitmap.format = format;
-  bitmap.rowstride = rowstride;
+  bmp = _cogl_bitmap_new_from_data ((guint8 *) data,
+                                    format,
+                                    width,
+                                    height,
+                                    rowstride,
+                                    NULL, NULL);
 
-  return cogl_texture_new_from_bitmap (&bitmap, flags, internal_format);
+  tex = cogl_texture_new_from_bitmap (bmp, flags, internal_format);
+
+  cogl_object_unref (bmp);
+
+  return tex;
 }
 
 CoglHandle
@@ -455,29 +474,29 @@ cogl_texture_new_from_file (const char        *filename,
                             CoglPixelFormat    internal_format,
                             GError           **error)
 {
-  CoglHandle bmp_handle;
   CoglBitmap *bmp;
   CoglHandle handle = COGL_INVALID_HANDLE;
+  CoglPixelFormat src_format;
 
   g_return_val_if_fail (error == NULL || *error == NULL, COGL_INVALID_HANDLE);
 
-  bmp_handle = cogl_bitmap_new_from_file (filename, error);
-  if (bmp_handle == COGL_INVALID_HANDLE)
+  bmp = cogl_bitmap_new_from_file (filename, error);
+  if (bmp == NULL)
     return COGL_INVALID_HANDLE;
 
-  bmp = (CoglBitmap *) bmp_handle;
+  src_format = _cogl_bitmap_get_format (bmp);
 
   /* We know that the bitmap data is solely owned by this function so
      we can do the premult conversion in place. This avoids having to
      copy the bitmap which will otherwise happen in
      _cogl_texture_prepare_for_upload */
-  internal_format = _cogl_texture_determine_internal_format (bmp->format,
-                                                             internal_format);
-  if (!_cogl_texture_needs_premult_conversion (bmp->format, internal_format) ||
-      _cogl_bitmap_convert_premult_status (bmp, bmp->format ^ COGL_PREMULT_BIT))
+  internal_format =
+    _cogl_texture_determine_internal_format (src_format, internal_format);
+  if (!_cogl_texture_needs_premult_conversion (src_format, internal_format) ||
+      _cogl_bitmap_convert_premult_status (bmp, src_format ^ COGL_PREMULT_BIT))
     handle = cogl_texture_new_from_bitmap (bmp, flags, internal_format);
 
-  cogl_handle_unref (bmp);
+  cogl_object_unref (bmp);
 
   return handle;
 }
@@ -540,7 +559,8 @@ cogl_texture_new_from_buffer_EXP (CoglHandle          buffer,
 {
   CoglHandle texture;
   CoglBuffer *cogl_buffer;
-  CoglPixelBuffer *pixel_buffer;
+  CoglPixelArray *pixel_array;
+  CoglBitmap *bmp;
 
   g_return_val_if_fail (cogl_is_buffer (buffer), COGL_INVALID_HANDLE);
 
@@ -548,54 +568,37 @@ cogl_texture_new_from_buffer_EXP (CoglHandle          buffer,
     return COGL_INVALID_HANDLE;
 
   cogl_buffer = COGL_BUFFER (buffer);
-  pixel_buffer = COGL_PIXEL_BUFFER (buffer);
+  pixel_array = COGL_PIXEL_ARRAY (buffer);
 
   /* Rowstride from CoglBuffer or even width * bpp if not given */
   if (rowstride == 0)
-    rowstride = pixel_buffer->stride;
+    rowstride = pixel_array->stride;
   if (rowstride == 0)
     rowstride = width * _cogl_get_format_bpp (format);
 
   /* use the CoglBuffer height and width as last resort */
   if (width == 0)
-    width = pixel_buffer->width;
+    width = pixel_array->width;
   if (height == 0)
-    height = pixel_buffer->height;
+    height = pixel_array->height;
   if (width == 0 || height == 0)
     {
       /* no width or height specified, neither at creation time (because the
-       * buffer was created by cogl_pixel_buffer_new()) nor when calling this
+       * array was created by cogl_pixel_array_new()) nor when calling this
        * function */
       return COGL_INVALID_HANDLE;
     }
 
-#if !defined (COGL_HAS_GLES)
-  if (cogl_features_available (COGL_FEATURE_PBOS))
-    {
-      CoglBitmap bitmap;
+  /* Wrap the buffer into a bitmap */
+  bmp = _cogl_bitmap_new_from_buffer (cogl_buffer,
+                                      format,
+                                      width, height,
+                                      rowstride,
+                                      offset);
 
-      /* Wrap the data into a bitmap */
-      bitmap.width = width;
-      bitmap.height = height;
-      bitmap.data = GUINT_TO_POINTER (offset);
-      bitmap.format = format;
-      bitmap.rowstride = rowstride;
+  texture = cogl_texture_new_from_bitmap (bmp, flags, internal_format);
 
-      _cogl_buffer_bind (cogl_buffer, GL_PIXEL_UNPACK_BUFFER);
-      texture =  cogl_texture_new_from_bitmap (&bitmap, flags, internal_format);
-      _cogl_buffer_bind (NULL, GL_PIXEL_UNPACK_BUFFER);
-    }
-  else
-#endif
-    {
-      texture = cogl_texture_new_from_data (width,
-			                    height,
-                                            flags,
-                                            format,
-                                            internal_format,
-                                            rowstride,
-                                            cogl_buffer->data);
-    }
+  cogl_object_unref (bmp);
 
   return texture;
 }
@@ -811,6 +814,45 @@ _cogl_texture_ensure_non_quad_rendering (CoglHandle handle)
   tex->vtable->ensure_non_quad_rendering (tex);
 }
 
+gboolean
+_cogl_texture_set_region_from_bitmap (CoglHandle    handle,
+                                      int           src_x,
+                                      int           src_y,
+                                      int           dst_x,
+                                      int           dst_y,
+                                      unsigned int  dst_width,
+                                      unsigned int  dst_height,
+                                      CoglBitmap   *bmp)
+{
+  CoglTexture *tex = COGL_TEXTURE (handle);
+  GLenum       closest_gl_format;
+  GLenum       closest_gl_type;
+  gboolean     ret;
+
+  /* Shortcut out early if the image is empty */
+  if (dst_width == 0 || dst_height == 0)
+    return TRUE;
+
+  /* Prepare the bitmap so that it will do the premultiplication
+     conversion */
+  bmp = _cogl_texture_prepare_for_upload (bmp,
+                                          cogl_texture_get_format (handle),
+                                          NULL,
+                                          NULL,
+                                          &closest_gl_format,
+                                          &closest_gl_type);
+
+  ret = tex->vtable->set_region (handle,
+                                 src_x, src_y,
+                                 dst_x, dst_y,
+                                 dst_width, dst_height,
+                                 bmp);
+
+  cogl_object_unref (bmp);
+
+  return ret;
+}
+
 gboolean
 cogl_texture_set_region (CoglHandle       handle,
 			 int              src_x,
@@ -825,21 +867,35 @@ cogl_texture_set_region (CoglHandle       handle,
 			 unsigned int     rowstride,
 			 const guint8    *data)
 {
-  CoglTexture *tex;
+  CoglBitmap *source_bmp;
+  gboolean    ret;
 
-  if (!cogl_is_texture (handle))
+  /* Check for valid format */
+  if (format == COGL_PIXEL_FORMAT_ANY)
     return FALSE;
 
-  tex = COGL_TEXTURE (handle);
+  /* Rowstride from width if none specified */
+  if (rowstride == 0)
+    rowstride = _cogl_get_format_bpp (format) * width;
 
-  return tex->vtable->set_region (tex,
-                                  src_x, src_y,
-                                  dst_x, dst_y,
-                                  dst_width, dst_height,
-                                  width, height,
-                                  format,
-                                  rowstride,
-                                  data);
+  /* Init source bitmap */
+  source_bmp = _cogl_bitmap_new_from_data ((guint8 *) data,
+                                           format,
+                                           width,
+                                           height,
+                                           rowstride,
+                                           NULL, /* destroy_fn */
+                                           NULL); /* destroy_fn_data */
+
+  ret = _cogl_texture_set_region_from_bitmap (handle,
+                                              src_x, src_y,
+                                              dst_x, dst_y,
+                                              dst_width, dst_height,
+                                              source_bmp);
+
+  cogl_object_unref (source_bmp);
+
+  return ret;
 }
 
 /* Reads back the contents of a texture by rendering it to the framebuffer
@@ -866,7 +922,7 @@ do_texture_draw_and_read (CoglHandle   handle,
   float       tx1, ty1;
   float       tx2, ty2;
   int         bw,  bh;
-  CoglBitmap  rect_bmp;
+  CoglBitmap  *rect_bmp;
   unsigned int  tex_width, tex_height;
 
   bpp = _cogl_get_format_bpp (COGL_PIXEL_FORMAT_RGBA_8888);
@@ -894,10 +950,19 @@ do_texture_draw_and_read (CoglHandle   handle,
       /* Walk X axis until whole bitmap width consumed */
       for (bw = tex_width; bw > 0; bw-=viewport[2])
         {
+          int width;
+          int height;
+          int rowstride;
+          guint8 *data;
+
           /* Rectangle X coords */
           rx1 = rx2;
           rx2 += (bw < viewport[2]) ? bw : viewport[2];
 
+          width = rx2 - rx1;
+          height = ry2 - ry1;
+          rowstride = width * bpp;
+
           /* Normalized texture X coords */
           tx1 = tx2;
           tx2 = (rx2 / (float) tex_width);
@@ -909,32 +974,35 @@ do_texture_draw_and_read (CoglHandle   handle,
                                               tx1, ty1,
                                               tx2, ty2);
 
-          /* Read into a temporary bitmap */
-          rect_bmp.format = COGL_PIXEL_FORMAT_RGBA_8888;
-          rect_bmp.width = rx2 - rx1;
-          rect_bmp.height = ry2 - ry1;
-          rect_bmp.rowstride = bpp * rect_bmp.width;
-          rect_bmp.data = g_malloc (rect_bmp.rowstride *
-                                    rect_bmp.height);
+          data = g_malloc (height * rowstride);
 
-          _cogl_texture_driver_prep_gl_for_pixels_download (rect_bmp.rowstride,
-                                                            bpp);
-          GE( glReadPixels (viewport[0], viewport[1],
-                            rect_bmp.width,
-                            rect_bmp.height,
-                            GL_RGBA, GL_UNSIGNED_BYTE,
-                            rect_bmp.data) );
+          /* Read into a temporary bitmap */
+          rect_bmp =
+            _cogl_bitmap_new_from_data (data,
+                                        COGL_PIXEL_FORMAT_RGBA_8888,
+                                        width,
+                                        height,
+                                        rowstride,
+                                        (CoglBitmapDestroyNotify) g_free,
+                                        NULL);
+
+          cogl_read_pixels (viewport[0], viewport[1],
+                            width,
+                            height,
+                            COGL_READ_PIXELS_COLOR_BUFFER,
+                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                            data);
 
           /* Copy to target bitmap */
-          _cogl_bitmap_copy_subregion (&rect_bmp,
+          _cogl_bitmap_copy_subregion (rect_bmp,
                                        target_bmp,
                                        0,0,
                                        rx1,ry1,
-                                       rect_bmp.width,
-                                       rect_bmp.height);
+                                       width,
+                                       height);
 
           /* Free temp bitmap */
-          g_free (rect_bmp.data);
+          cogl_object_unref (rect_bmp);
         }
     }
 }
@@ -952,12 +1020,15 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                              GLuint       target_gl_type)
 {
   int        bpp;
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int        viewport[4];
-  CoglBitmap alpha_bmp;
+  CoglBitmap *alpha_bmp;
   CoglHandle prev_source;
   CoglMatrixStack *projection_stack;
   CoglMatrixStack *modelview_stack;
+  int target_width = _cogl_bitmap_get_width (target_bmp);
+  int target_height = _cogl_bitmap_get_height (target_bmp);
+  int target_rowstride = _cogl_bitmap_get_rowstride (target_bmp);
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
@@ -970,8 +1041,8 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
       viewport[2] <= 0 || viewport[3] <= 0)
     return FALSE;
 
-  /* Setup orthographic projection into current viewport (0,0 in bottom-left
-   * corner to draw the texture upside-down so we match the way glReadPixels
+  /* Setup orthographic projection into current viewport (0,0 in top-left
+   * corner to draw the texture upside-down so we match the way cogl_read_pixels
    * works)
    */
 
@@ -980,7 +1051,7 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
   _cogl_matrix_stack_load_identity (projection_stack);
   _cogl_matrix_stack_ortho (projection_stack,
                             0, (float)(viewport[2]),
-                            0, (float)(viewport[3]),
+                            (float)(viewport[3]), 0,
                             (float)(0),
                             (float)(100));
 
@@ -1008,6 +1079,10 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                                    "RGBA = REPLACE (TEXTURE)",
                                    NULL);
 
+  cogl_material_set_layer_filters (ctx->texture_download_material, 0,
+                                   COGL_MATERIAL_FILTER_NEAREST,
+                                   COGL_MATERIAL_FILTER_NEAREST);
+
   do_texture_draw_and_read (handle, target_bmp, viewport);
 
   /* Check whether texture has alpha and framebuffer not */
@@ -1030,14 +1105,22 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
       guint8 *srcpixel;
       guint8 *dstpixel;
       int     x,y;
+      int     alpha_rowstride = bpp * target_width;
+
+      if ((dstdata = _cogl_bitmap_map (target_bmp,
+                                       COGL_BUFFER_ACCESS_WRITE,
+                                       COGL_BUFFER_MAP_HINT_DISCARD)) == NULL)
+        return FALSE;
+
+      srcdata = g_malloc (alpha_rowstride * target_height);
 
       /* Create temp bitmap for alpha values */
-      alpha_bmp.format = COGL_PIXEL_FORMAT_RGBA_8888;
-      alpha_bmp.width = target_bmp->width;
-      alpha_bmp.height = target_bmp->height;
-      alpha_bmp.rowstride = bpp * alpha_bmp.width;
-      alpha_bmp.data = g_malloc (alpha_bmp.rowstride *
-                                 alpha_bmp.height);
+      alpha_bmp = _cogl_bitmap_new_from_data (srcdata,
+                                              COGL_PIXEL_FORMAT_RGBA_8888,
+                                              target_width, target_height,
+                                              alpha_rowstride,
+                                              (CoglBitmapDestroyNotify) g_free,
+                                              NULL);
 
       /* Draw alpha values into RGB channels */
       cogl_material_set_layer_combine (ctx->texture_download_material,
@@ -1045,25 +1128,25 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                                        "RGBA = REPLACE (TEXTURE[A])",
                                        NULL);
 
-      do_texture_draw_and_read (handle, &alpha_bmp, viewport);
+      do_texture_draw_and_read (handle, alpha_bmp, viewport);
 
       /* Copy temp R to target A */
-      srcdata = alpha_bmp.data;
-      dstdata = target_bmp->data;
 
-      for (y=0; y<target_bmp->height; ++y)
+      for (y=0; y<target_height; ++y)
         {
-          for (x=0; x<target_bmp->width; ++x)
+          for (x=0; x<target_width; ++x)
             {
               srcpixel = srcdata + x*bpp;
               dstpixel = dstdata + x*bpp;
               dstpixel[3] = srcpixel[0];
             }
-          srcdata += alpha_bmp.rowstride;
-          dstdata += target_bmp->rowstride;
+          srcdata += alpha_rowstride;
+          dstdata += target_rowstride;
         }
 
-      g_free (alpha_bmp.data);
+      _cogl_bitmap_unmap (target_bmp);
+
+      cogl_object_unref (alpha_bmp);
     }
 
   /* Restore old state */
@@ -1083,13 +1166,134 @@ cogl_texture_get_data (CoglHandle       handle,
 		       unsigned int     rowstride,
 		       guint8          *data)
 {
-  CoglTexture *tex;
+  CoglTexture     *tex;
+  int              bpp;
+  int              byte_size;
+  CoglPixelFormat  closest_format;
+  int              closest_bpp;
+  GLenum           closest_gl_format;
+  GLenum           closest_gl_type;
+  CoglBitmap      *target_bmp;
+  CoglBitmap      *new_bmp;
+  gboolean         success;
+  guint8          *src;
+  guint8          *dst;
+  int              y;
+  int              tex_width;
+  int              tex_height;
 
   if (!cogl_is_texture (handle))
-    return FALSE;
+    return 0;
 
   tex = COGL_TEXTURE (handle);
 
-  return tex->vtable->get_data (handle, format, rowstride, data);
-}
+  /* Default to internal format if none specified */
+  if (format == COGL_PIXEL_FORMAT_ANY)
+    format = cogl_texture_get_format (handle);
 
+  tex_width = cogl_texture_get_width (handle);
+  tex_height = cogl_texture_get_height (handle);
+
+  /* Rowstride from texture width if none specified */
+  bpp = _cogl_get_format_bpp (format);
+  if (rowstride == 0)
+    rowstride = tex_width * bpp;
+
+  /* Return byte size if only that requested */
+  byte_size = tex_height * rowstride;
+  if (data == NULL)
+    return byte_size;
+
+  closest_format =
+    _cogl_texture_driver_find_best_gl_get_data_format (format,
+                                                       &closest_gl_format,
+                                                       &closest_gl_type);
+  closest_bpp = _cogl_get_format_bpp (closest_format);
+
+  /* Is the requested format supported? */
+  if (closest_format == format)
+    /* Target user data directly */
+    target_bmp = _cogl_bitmap_new_from_data (data,
+                                             format,
+                                             tex_width,
+                                             tex_height,
+                                             rowstride,
+                                             NULL, NULL);
+  else
+    {
+      int target_rowstride = tex_width * closest_bpp;
+      guint8 *target_data = g_malloc (tex_height * target_rowstride);
+      target_bmp = _cogl_bitmap_new_from_data (target_data,
+                                               closest_format,
+                                               tex_width,
+                                               tex_height,
+                                               target_rowstride,
+                                               (CoglBitmapDestroyNotify) g_free,
+                                               NULL);
+    }
+
+  if ((dst = _cogl_bitmap_map (target_bmp, COGL_BUFFER_ACCESS_WRITE,
+                               COGL_BUFFER_MAP_HINT_DISCARD)) == NULL)
+    {
+      cogl_object_unref (target_bmp);
+      return 0;
+    }
+
+  success = tex->vtable->get_data (tex,
+                                   closest_format,
+                                   rowstride,
+                                   dst);
+
+  _cogl_bitmap_unmap (target_bmp);
+
+  /* XXX: In some cases _cogl_texture_2d_download_from_gl may fail
+   * to read back the texture data; such as for GLES which doesn't
+   * support glGetTexImage, so here we fallback to drawing the
+   * texture and reading the pixels from the framebuffer. */
+  if (!success)
+    _cogl_texture_draw_and_read (tex, target_bmp,
+                                 closest_gl_format,
+                                 closest_gl_type);
+
+  /* Was intermediate used? */
+  if (closest_format != format)
+    {
+      guint8 *new_bmp_data;
+      int new_bmp_rowstride;
+
+      /* Convert to requested format */
+      new_bmp = _cogl_bitmap_convert_format_and_premult (target_bmp,
+                                                         format);
+
+      /* Free intermediate data and return if failed */
+      cogl_object_unref (target_bmp);
+
+      if (new_bmp == NULL)
+        return 0;
+
+      new_bmp_rowstride = _cogl_bitmap_get_rowstride (new_bmp);
+      new_bmp_data = _cogl_bitmap_map (new_bmp, COGL_BUFFER_ACCESS_WRITE,
+                                       COGL_BUFFER_MAP_HINT_DISCARD);
+
+      if (new_bmp_data == NULL)
+        {
+          cogl_object_unref (new_bmp);
+          return 0;
+        }
+
+      /* Copy to user buffer */
+      for (y = 0; y < tex_height; ++y)
+        {
+          src = new_bmp_data + y * new_bmp_rowstride;
+          dst = data + y * rowstride;
+          memcpy (dst, src, tex_width * bpp);
+        }
+
+      _cogl_bitmap_unmap (new_bmp);
+
+      /* Free converted data */
+      cogl_object_unref (new_bmp);
+    }
+
+  return byte_size;
+}
