@@ -34,11 +34,9 @@
 #include <cogl/cogl-object.h>
 #include <cogl/cogl-bitmap.h>
 #include <cogl/cogl-color.h>
-#include <cogl/cogl-debug.h>
 #include <cogl/cogl-fixed.h>
 #include <cogl/cogl-material.h>
 #include <cogl/cogl-matrix.h>
-#include <cogl/cogl-vector.h>
 #include <cogl/cogl-offscreen.h>
 #include <cogl/cogl-primitives.h>
 #include <cogl/cogl-path.h>
@@ -53,8 +51,9 @@
 
 #if defined (COGL_ENABLE_EXPERIMENTAL_API)
 #include <cogl/cogl-buffer.h>
-#include <cogl/cogl-pixel-buffer.h>
+#include <cogl/cogl-pixel-array.h>
 #include <cogl/cogl-vector.h>
+#include <cogl/cogl-texture-3d.h>
 #endif
 
 G_BEGIN_DECLS
@@ -66,6 +65,8 @@ G_BEGIN_DECLS
  * General utility functions for COGL.
  */
 
+typedef struct _CoglFramebuffer CoglFramebuffer;
+
 /**
  * cogl_get_option_group:
  *
@@ -173,7 +174,7 @@ cogl_get_bitmasks (int *red,
  * Since: 1.4
  */
 int
-_cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer);
 
 /*
  * _cogl_framebuffer_get_green_bits:
@@ -186,7 +187,7 @@ _cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-_cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer);
 
 /*
  * _cogl_framebuffer_get_blue_bits:
@@ -199,7 +200,7 @@ _cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-_cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer);
 
 /*
  * _cogl_framebuffer_get_alpha_bits:
@@ -212,7 +213,7 @@ _cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-_cogl_framebuffer_get_alpha_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer);
 
 /**
  * cogl_perspective:
@@ -258,10 +259,10 @@ cogl_frustum (float left,
  * @right: The coordinate for the right clipping plane
  * @bottom: The coordinate for the bottom clipping plane
  * @top: The coordinate for the top clipping plane
- * @near: The <b>distance</b> to the near clipping plane (negative if
- *        the plane is behind the viewer)
- * @far: The <b>distance</b> for the far clipping plane (negative if
- *       the plane is behind the viewer)
+ * @near: The <emphasis>distance</emphasis> to the near clipping
+ *   plane (negative if the plane is behind the viewer)
+ * @far: The <emphasis>distance</emphasis> for the far clipping
+ *   plane (negative if the plane is behind the viewer)
  *
  * Replaces the current projection matrix with an orthographic projection
  * matrix. See <xref linkend="cogl-ortho-matrix"/> to see how the matrix is
@@ -272,7 +273,7 @@ cogl_frustum (float left,
  *   <graphic fileref="cogl_ortho.png" format="PNG"/>
  * </figure>
  *
- * <note>cogl_ortho copies the arguments from OpenGL's glOrtho even
+ * <note>This function copies the arguments from OpenGL's glOrtho() even
  * though they are unnecessarily confusing due to the z near and z far
  * arguments actually being a "distance" from the origin, where
  * negative values are behind the viewer, instead of coordinates for
@@ -911,8 +912,7 @@ cogl_clip_stack_restore (void) G_GNUC_DEPRECATED;
 
 /**
  * cogl_set_framebuffer:
- * @buffer: The #CoglHandle of a Cogl framebuffer; either onscreen or
- *          offscreen.
+ * @buffer: A #CoglFramebuffer object, either onscreen or offscreen.
  *
  * This redirects all subsequent drawing to the specified framebuffer. This can
  * either be an offscreen buffer created with cogl_offscreen_new_to_texture ()
@@ -921,12 +921,11 @@ cogl_clip_stack_restore (void) G_GNUC_DEPRECATED;
  * Since: 1.2
  */
 void
-cogl_set_framebuffer (CoglHandle buffer);
+cogl_set_framebuffer (CoglFramebuffer *buffer);
 
 /**
  * cogl_push_framebuffer:
- * @buffer: The #CoglHandle of a Cogl framebuffer; either onscreen or
- *          offscreen.
+ * @buffer: A #CoglFramebuffer object, either onscreen or offscreen.
  *
  * Redirects all subsequent drawing to the specified framebuffer. This can
  * either be an offscreen buffer created with cogl_offscreen_new_to_texture ()
@@ -980,7 +979,7 @@ cogl_set_framebuffer (CoglHandle buffer);
  *
  * static void
  * my_init_framebuffer (ClutterStage *stage,
- *                      CoglHandle framebuffer,
+ *                      CoglFramebuffer *framebuffer,
  *                      unsigned int framebuffer_width,
  *                      unsigned int framebuffer_height)
  * {
@@ -1003,7 +1002,7 @@ cogl_set_framebuffer (CoglHandle buffer);
  * Since: 1.2
  */
 void
-cogl_push_framebuffer (CoglHandle buffer);
+cogl_push_framebuffer (CoglFramebuffer *buffer);
 
 /**
  * cogl_pop_framebuffer:
