@@ -28,7 +28,7 @@
 #include "cogl-context.h"
 #include "cogl-gles2-wrapper.h"
 
-#define COGL_FEATURE_BEGIN(a, b, c, d, e, f)
+#define COGL_FEATURE_BEGIN(a, b, c, d, e, f, g)
 #define COGL_FEATURE_FUNCTION(ret, name, args) \
   _context->drv.pf_ ## name = NULL;
 #define COGL_FEATURE_END()
@@ -40,7 +40,7 @@ _cogl_create_context_driver (CoglContext *_context)
 
   /* Init the GLES2 wrapper */
 #ifdef HAVE_COGL_GLES2
-  cogl_gles2_wrapper_init (&_context->drv.gles2);
+  _cogl_gles2_wrapper_init (&_context->drv.gles2);
 #endif
 }
 
