@@ -26,6 +26,7 @@
 
 #include "cogl.h" /* needed for gl header include */
 #include "cogl-internal.h"
+#include "cogl-program.h"
 
 G_BEGIN_DECLS
 
@@ -44,9 +45,6 @@ typedef struct _CoglGles2WrapperSettings  CoglGles2WrapperSettings;
 typedef struct _CoglGles2WrapperProgram	  CoglGles2WrapperProgram;
 typedef struct _CoglGles2WrapperShader	  CoglGles2WrapperShader;
 
-#define COGL_GLES2_NUM_CUSTOM_UNIFORMS    16
-#define COGL_GLES2_UNBOUND_CUSTOM_UNIFORM -2
-
 /* Must be a power of two */
 #define COGL_GLES2_MODELVIEW_STACK_SIZE   32
 #define COGL_GLES2_PROJECTION_STACK_SIZE  2
@@ -128,6 +126,10 @@ struct _CoglGles2WrapperTexEnv
   GLfloat   texture_combine_constant[4];
 
   GLboolean point_sprite_coords;
+
+  /* Texture target that was last enabled for this unit. This assumes
+     that there can only be one target enabled per unit */
+  GLenum    texture_target;
 };
 
 /* NB: We get a copy of this for each fragment/vertex
@@ -186,7 +188,7 @@ struct _CoglGles2Wrapper
   /* Whether the settings have changed since the last draw */
   gboolean settings_dirty;
   /* Uniforms that have changed since the last draw */
-  int dirty_uniforms, dirty_custom_uniforms;
+  int dirty_uniforms;
 
   /* Attribute pointers that have changed since the last draw */
   int dirty_attribute_pointers;
@@ -210,13 +212,14 @@ struct _CoglGles2Wrapper
   GLfloat fog_end;
   GLfloat fog_color[4];
   GLfloat point_size;
-  CoglBoxedValue custom_uniforms[COGL_GLES2_NUM_CUSTOM_UNIFORMS];
 };
 
 struct _CoglGles2WrapperProgram
 {
   GLuint    program;
 
+  CoglProgram *user_program;
+
   /* The settings that were used to generate this combination */
   CoglGles2WrapperSettings settings;
 
@@ -226,7 +229,7 @@ struct _CoglGles2WrapperProgram
 
   /* The uniforms for this program */
   CoglGles2WrapperUniforms uniforms;
-  GLint custom_uniforms[COGL_GLES2_NUM_CUSTOM_UNIFORMS];
+  GLint custom_gl_uniforms[COGL_PROGRAM_NUM_CUSTOM_UNIFORMS];
 };
 
 struct _CoglGles2WrapperShader
